﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO; 
using Arction.LightningChartUltimate;
using Arction.LightningChartUltimate.Axes;
using Arction.LightningChartUltimate.SeriesXY;
using Arction.LightningChartUltimate.Series3D;
using Arction.LightningChartUltimate.Views;
using Arction.LightningChartUltimate.Views.ViewXY;
using Arction.LightningChartUltimate.Views.View3D;

namespace IntensityOn3DSurface
{
    public partial class Form1 : Form
    {
        LightningChartUltimate _intensityChart;
        LightningChartUltimate _surface3DChart;

        const int IntensityRowCount = 200;
        const int IntensityColumnCount = 200;

        const int SurfaceColumnCount = 300;
        const int SurfaceRowCount = 300; 

        int _intensityUpdateRound = 0;
        int _surfaceUpdateRound = 0; 
        bool _intensityBitmapUpdated = false;
        MemoryStream _bitmapStream = new MemoryStream();  

        public Form1()
        {
            InitializeComponent();
            CreateIntensityChart();
            SetNewIntensityData(); 

            Create3DSurfaceChart();
           
            timerIntensityDataUpdater.Start();
            timerSurface3DGeometryUpdater.Start(); 
        }

        /// <summary>
        /// Intensity chart renders intensity data from given data set, and exports a bitmap image. 
        /// </summary>
        void CreateIntensityChart()
        {
            _intensityChart = new LightningChartUltimate();
            
            _intensityChart.BeginUpdate(); 

            _intensityChart.Parent = this;
            _intensityChart.Dock = DockStyle.None;
            _intensityChart.Width = IntensityColumnCount;
            _intensityChart.Height = IntensityRowCount;
            _intensityChart.RenderOptions.AntiAliasLevel = 0; 

            _intensityChart.ActiveView = ActiveView.ViewXY;
            _intensityChart.Title.Visible = false; 
           
            ViewXY v = _intensityChart.ViewXY;
            v.LegendBox.Visible = false; 

            //Remove all margins 
            v.Margins = new System.Windows.Forms.Padding(0);
            v.GraphBorderColor = Color.Transparent;

            IntensityGridSeries grid = new IntensityGridSeries(v, v.XAxes[0], v.YAxes[0]);
            grid.SetRangesXY(0, 100, 0, 100);
            grid.SetSize(IntensityColumnCount, IntensityRowCount);
            grid.ContourLineType = ContourLineType.None;
            

            v.IntensityGridSeries.Add(grid);
            
            //Set axis ranges sow that intensity grid fills it. Minimums same and Maximums same. 
            v.XAxes[0].SetRange(0, 100);
            v.YAxes[0].SetRange(0, 100); 
            
            _intensityChart.EndUpdate(); 
        }

        private void timerIntensityDataUpdater_Tick(object sender, EventArgs e)
        {
            if (_intensityChart != null)
            {
                SetNewIntensityData();
            }
        }

        void SetNewIntensityData()
        {
            _intensityChart.BeginUpdate(); 
            
            IntensityGridSeries grid =_intensityChart.ViewXY.IntensityGridSeries[0]; 
            IntensityPoint[,] data = grid.Data;
            Random rand = new Random(); 
            
            Parallel.For(0, IntensityColumnCount, ((row) =>
            {
                

                for (int col = 0; col < IntensityColumnCount; col++)
                {
                    data[col, row].Value = Math.Sin((double)col * (double)_intensityUpdateRound / (double)IntensityColumnCount) * 30
                        + Math.Sin((double)row * (double)_intensityUpdateRound / (double)IntensityRowCount) * 30;
                }
            }));

           
            grid.InvalidateValuesDataOnly();
            
            _intensityChart.EndUpdate();
            _intensityChart.SaveToStream(_bitmapStream, TargetImageFormat.Bmp); 
           
            _intensityBitmapUpdated = true;
            _intensityUpdateRound++;
            if (_intensityUpdateRound == 20)
                _intensityUpdateRound = 0;
        }


        /// <summary>
        /// Intensity chart renders intensity data from given data set, and exports a bitmap image. 
        /// </summary>
        void Create3DSurfaceChart()
        {
            _surface3DChart = new LightningChartUltimate();

            _surface3DChart.BeginUpdate();

            _surface3DChart.Parent = this;
            _surface3DChart.Dock = DockStyle.Fill;
            _surface3DChart.BringToFront(); 

            _surface3DChart.ActiveView = ActiveView.View3D;

            View3D v = _surface3DChart.View3D;

            v.LegendBox.Visible = false; 

            SurfaceGridSeries3D surface = new SurfaceGridSeries3D(v, Axis3DBinding.Primary, Axis3DBinding.Primary, Axis3DBinding.Primary);
            
            surface.SetRangesXZ(-500, 500, -500, 500);
            surface.SetSize(SurfaceColumnCount, SurfaceRowCount);
            surface.BitmapFill.Image = Bitmap.FromStream(_bitmapStream); 
            surface.Fill = SurfaceFillStyle.Bitmap;
            surface.ContourLineType = ContourLineType.None;
            surface.WireframeType = SurfaceWireframeType.None;
            surface.ColorSaturation = 90;
            
            v.SurfaceGridSeries3D.Add(surface);

            //Set axis ranges sow that intensity grid fills it. Minimums same and Maximums same. 
            v.XAxisPrimary3D.SetRange(-500, 500);
            v.ZAxisPrimary3D.SetRange(-500, 500);
            v.YAxisPrimary3D.SetRange(0, 100);

            _surface3DChart.EndUpdate();
        }

        private void timerSurface3DGeometryUpdater_Tick(object sender, EventArgs e)
        {

            if (_surface3DChart != null)
            {
                _surface3DChart.BeginUpdate();

                SurfaceGridSeries3D surface = _surface3DChart.View3D.SurfaceGridSeries3D[0];
                SurfacePoint[,] data = surface.Data; 

                //Update grid's Y values 
                Parallel.For(0, SurfaceColumnCount, ((row) =>
                {

                    for (int col = 0; col < SurfaceColumnCount; col++)
                    {
                        data[col, row].Y = 50 + Math.Sin((double)col * (double)_surfaceUpdateRound / (double)SurfaceColumnCount / 5.0) * 20
                            + Math.Sin((double)row * (double)_surfaceUpdateRound / (double)SurfaceRowCount / 5.0) * 20;
                    }
                }));


                //Update the intensity fill bitmap, if it's been updated 
                if (_intensityBitmapUpdated)
                {
                    surface.BitmapFill.Image = Bitmap.FromStream(_bitmapStream);
                    _intensityBitmapUpdated = false; 
                }

                surface.InvalidateData(); 

                _surface3DChart.EndUpdate();

                _surfaceUpdateRound++;
                if(_surfaceUpdateRound > 100)
                    _surfaceUpdateRound = 0;
            }
        }
    }
}
